# -*- coding: utf-8 -*-
#	Parsing a fasta file.
#	Author - Janu Verma
#	jv367@cornell.edu

import sys
from sequenceOperations import SequenceManipulation
from trimming import Trimming


class FastaParser:
	"""
	Parses a FASTA file to extract the sequences and header information, if any. 

	Parameters
	----------
	fasta_file : Fasta file to be parsed. 


	Example
	-------
	>>> import sys
	>>> input_file = sys.argv[1] 
	>>> out = FastaParser(input_file)
	>>> seqDict = out.sequenceDict()
	>>> print len(seqDict.keys())
	"""

	def __init__(self, fasta_file):
		self.ff = fasta_file
		


	def readFasta(self, fastaFile):
		"""
		Reads and parser the FASTA file. 

		Parameters
		----------
		fastaFile - A FASTA file.

		Returns
		------
		Generator object containing sequences. 
		"""	
		name, seq = None, []
		for line in fastaFile:
			line = line.rstrip()
			if (line.startswith(">")):
				if name: yield (name, ''.join(seq))
				name, seq = line, []
			else:
				seq.append(line)
		if name: yield (name, ''.join(seq))




	def sequenceDict(self):
		"""
		Creates a dictionary of sequences with their header.

		Returns
		-------
		A dictionary of sequences.

		"""
		with open(self.ff) as fastaFile:
			sequences = {}
			for name, seq in self.readFasta(fastaFile):
				sequences[name] = seq
		return sequences



	def seqNames(self):
		"""
		Names/Headers of all the sequences.

		Returns
		-------
		A list of names of all the sequences in the FASTA file. 
		"""
		seqDict = self.sequenceDict()
		return seqDict.keys()



	def seqFromName(self, name):
		"""
		Extract the sequence corresponding to the given name. 

		Parameters
		---------
		name : Name of the sequence to be retrieved. 

		Returns
		-------
		Sequence corresponding to the input name. 
		"""
		seqDict = self.sequenceDict()
		return seqDict[name]



	def reverseComplement(self, nameSeq):
		"""
		Compute the reverse complement of a given sequence. 

		Parameters
		----------
		sequence: Name of the sequence whose reverse complement is to be computed. 

		Returns
		-------
		sequence which is the reverse complement of the input sequence.
		"""
		seqDict = self.sequenceDict()
		sequence = seqDict[nameSeq]
		new_seq = SequenceManipulation(sequence)
		return new_seq.reverseComplement()




	def maskSeq(self, name, interval, toLower=False, maskingChar='N'):
		"""
		Masks the sequence based on the given interval. 

		Parameters
		---------
		name: Name/header of the sequence.  
		interval: A tuple containing the start and end positions for the masking. 
		toLower: If True, the sequence in the interval is converted to lower case bases.
						Default is False. 
		maskingChar :  Masking character. Default is 'N'.

		Returns
		-------
		Masked sequence.
		"""
		seqDict = self.sequenceDict()
		sequence = seqDict[name]
		masker = SequenceManipulation(sequence)
		return masker.maskSequence(interval, toLower=False, maskingChar='N')




	def trimSeq(self, name, interval, quality = None):
		"""
		Trims the sequence from both sides based on the interval.

		Parameters
		----------
		name : Name/header of the sequence to be trimmed.
		interval : The interval containing the number of bp's to be trimmed from left and right side respectively.

		Returns
		-------
		Trimmed sequence.			
		"""
		seqDict = self.sequenceDict()
		sequence = seqDict[name]
		trimmer = Trimming(sequence, quality)
		return trimmer.trimSequence(interval)





	def maskAll(self, intervals, toLower=False, maskingChar='N'):
		"""
		Masks the sequences in the FASTA file based on the given intervals. 

		Parameters
		--------- 
		intervals: A list of tuples containing the start and end positions for the masking. 
		toLower: If True, the sequence in the interval is converted to lower case bases.
						Default is False. 
		maskingChar :  Masking character. Default is 'N'.

		Returns
		-------
		Masked sequences.
		"""
		seqDict = self.sequenceDict()
		for i in range(len(seqDict.keys())):
			x = seqDict.keys()[i]
			interval = intervals[i]
			print self.maskSeq(x, interval, toLower=False, maskingChar='N')




	def trimAll(self, intervals, quality=None):
		"""
		Trims all the sequence in the FASTA file from both sides based on the intervals.

		Parameters
		----------
		interval : A list of tuples containing the number of bp's to be trimmed from left and right side respectively.

		Returns
		-------
		Trimmed sequences.			
		"""
		seqDict = self.sequenceDict()
		for i in range(len(seqDict.keys())):
			x = seqDict.keys()[i]
			interval = intervals[i]
			print self.trimSeq(x, interval, quality = None)





	def reverseComplementAll(self):
		"""
		Compute the reverse complements of all the sequences in the given FASTA file. 

		Parameters
		----------
		sequence: Name of the sequence whose reverse complement is to be computed. 

		Returns
		-------
		Prints the reverse complements.
		"""
		seqDict = self.sequenceDict()
		for i in range(len(seqDict.keys())):
			x = seqDict.keys()[i]
			print self.reverseComplement(x)




























# -*- coding: utf-8 -*-
#	Manipulating Sequences.
#	Author - Janu Verma
#	jv367@cornell.edu



class SequenceManipulation:
	"""
	Edits/Modifies a DNA sequence.

	Parameters
	----------
	input_sequence : A nucleotide sequence.  
	"""
	def __init__(self, input_sequence):
		self.seq = input_sequence

	def maskSequence(self, interval, toLower=False, maskingChar='N'):
		"""
		Masks the sequence based on the interval. 

		Parameters
		---------
		interval : A tuple containing the start and end positions for the masking. 
		toLower: If True, the sequence in the interval is converted to lower case bases.
						Default is False. 
		maskingChar :  Masking character. Default is 'N'.

		Returns
		-------
		Masked sequence.
		"""	
		sequence = self.seq
		start,end = interval

		if (toLower):
			return sequence[:start] + sequence[start:end].lower() + sequence[end:]
		else:
			return sequence[:start] + ''.join([maskingChar for x in range(end- start)]) + sequence[end:]


	def reverseComplement(self):
		"""
		Compute the reverse complement of a given sequence. 

		Returns
		-------
		sequence which is the reverse complement of the input sequence.
		"""	
		complementDict = {'A':'T', 'T':'A', 'G':'C', 'C':'G', 'N':'N'}
		result = ''
		for x in self.seq[::-1]:
			result += complementDict[x]
		return result	






















#!/usr/bin/env python
from setuptools import setup, find_packages
import os
import sys

sys.path.insert(0,'fastAQ')

sys.path.pop(0)

packages = ['fastAQ']

setup(
    name='fastaq',
    version='0.20',
    author='Janu Verma',
    author_email='jv367@cornell.edu',
    description='fastAQ is a very and super lightweight package for working with FASTA/FASTQ sequences',
    url='https://github.com/Jverma/fastAQ',
    platforms=['Linux','Mac OSX', 'Windows', 'Unix'],
    keywords=['Genomics','Quantitative genetics', 'python'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        #'License ::MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    packages=packages,
    #package_data={'TextGraphics':['Data/*.txt']},
    license='MIT'
    )			


# -*- coding: utf-8 -*-
#	Parsing a fastq file.
#	Author - Janu Verma
#	jv367@cornell.edu

import sys
from trimming import Trimming
from sequenceOperations import SequenceManipulation

class FastqParser:
	"""
	Parses a FASTQ file to extract the sequences and the base qualities. 

	Parameters
	----------
	fasta_file : Fastq file to be parsed. 


	Example
	-------
	>>> import sys
	>>> input_file = sys.argv[1] 
	>>> out = FastqParser(input_file)
	>>> seqDict = out.sequenceDict()
	>>> print len(seqDict.keys())
	"""

	def __init__(self, fastq_file):
		self.ff = fastq_file
		
	def readFastq(self, fastqFile):
		"""
		Reads and parser the FASTQ file. 

		Parameters
		----------
		fastqFile - A FASTQ file.

		Returns
		------
		Generator object containing sequences. 
		"""
		i = 1
		name, seq, baseQ = None, [], []
		for line in fastqFile:
			if (line.startswith("@")) and (i%4 != 0):
				if name: yield (name, ''.join(seq), ''.join(baseQ))
				name, seq, baseQ = line, [], []
			if (line[0] in ['A', 'G', 'T', 'C', 'N']):
				seq.append(line)
			if (i%4 == 0):
				baseQ.append(line)
			i += 1
		if name: yield (name, ''.join(seq), ''.join(baseQ))	




	def sequenceDict(self):
		"""
		Creates a dictionary of sequences with their header.

		Returns
		-------
		A dictionary of sequences.
		"""
		with open(self.ff) as fastaFile:
			sequences = {}
			for name,seq,baseQ in self.readFastq(fastaFile):
				sequences[name.strip()] = seq.strip()
			return sequences
	



	def baseQualities(self):
		"""
		Creates a dictionary of base qualities of the sequences.

		Returns
		-------
		A dictionary of base qualities.
		"""
		with open(self.ff) as fastaFile:
			qualities = {}
			for name,seq,baseQ in self.readFastq(fastaFile):
				qualities[name.strip()] = baseQ.strip()
			return qualities



	def seqNames(self):
		"""
		Names/Headers of all the sequences.

		Returns
		-------
		A list of names of all the sequences in the FASTQ file. 
		"""
		seqDict = self.sequenceDict()
		return seqDict.keys()


	def trimSeq(self, name, qualityCutOff=0, byInterval=False, interval=None, mott=False, limitValue=None):
		"""
		Trims the sequence.

		Parameters
		----------
		name : Name/header of the sequence to be trimmed.
		qualityCutOff : Threshold value of the quality for trimming sequence based on removing low quality bases.
		byInterval : If True, the sequence will be trimmed by removing bases according to the given interval.
		interval : The interval containing the number of bp's to be trimmed from left and right side respectively.
					Need byInterval to be True.
		mott : If True, the sequence will be trimmed according to the Mott's algorithm.
		limitValue : Numerical value of the limit to be used in Mott's algorithm. 
					Requires mott to be True.			 
		
		Returns
		-------
		Trimmed sequence.			
		"""
		seqDict = self.sequenceDict()
		qualDict = self.baseQualities()
		sequence = seqDict[name]
		quality = qualDict[name]
		trimmer = Trimming(sequence, quality)
		if (byInterval):
			return trimmer.trimSequence(interval)
		elif (mott):
			return trimmer.mott(limitValue)
		else:
			return trimmer.lowQualTrim(qualityCutOff)	


	def maskSeq(self, name, interval, toLower=False, maskingChar='N'):
		"""
		Masks the sequence based on the given interval. 

		Parameters
		---------
		name: Name/header of the sequence.  
		interval: A tuple containing the start and end positions for the masking. 
		toLower: If True, the sequence in the interval is converted to lower case bases.
						Default is False. 
		maskingChar :  Masking character. Default is 'N'.

		Returns
		-------
		Masked sequence.
		"""
		seqDict = self.sequenceDict()
		sequence = seqDict[name]
		masker = SequenceManipulation(sequence)
		return masker.maskSequence(interval, toLower=False, maskingChar='N')

					

	def maskAll(self, intervals, toLower=False, maskingChar='N'):
		"""
		Masks the sequences in the FASTA file based on the given intervals. 

		Parameters
		--------- 
		intervals: A list of tuples containing the start and end positions for the masking. 
		toLower: If True, the sequence in the interval is converted to lower case bases.
						Default is False. 
		maskingChar :  Masking character. Default is 'N'.

		Returns
		-------
		Masked sequences.
		"""
		seqDict = self.sequenceDict()
		for i in range(len(seqDict.keys())):
			x = seqDict.keys()[i]
			interval = intervals[i]
			print self.maskSeq(x, interval, toLower=False, maskingChar='N')
	



	def trimAll(self, qualityCutOff=0, byInterval=False, intervals=None, mott=False, limitValue=None):
		"""
		Trims all the sequence in the FASTA file from both sides based on the intervals.

		Parameters
		----------
		interval : A list of tuples containing the number of bp's to be trimmed from left and right side respectively.

		Returns
		-------
		Trimmed sequences.			
		"""
		seqDict = self.sequenceDict()
		for i in range(len(seqDict.keys())):
			x = seqDict.keys()[i]
			interval = intervals[i]
			print self.trimSeq(x, qualityCutOff=0, byInterval=False, interval=None, mott=False, limitValue=None)


	def reverseComplement(self, nameSeq):
		"""
		Compute the reverse complement of a given sequence. 

		Parameters
		----------
		sequence: Name of the sequence whose reverse complement is to be computed. 

		Returns
		-------
		sequence which is the reverse complement of the input sequence.
		"""
		seqDict = self.sequenceDict()
		sequence = seqDict[nameSeq]
		new_seq = SequenceManipulation(sequence)
		return new_seq.reverseComplement()



	def reverseComplementAll(self):
		"""
		Compute the reverse complements of all the sequences in the given FASTA file. 

		Parameters
		----------
		sequence: Name of the sequence whose reverse complement is to be computed. 

		Returns
		-------
		Prints the reverse complements.
		"""
		seqDict = self.sequenceDict()
		for i in range(len(seqDict.keys())):
			x = seqDict.keys()[i]
			print self.reverseComplement(x)	







			




















# -*- coding: utf-8 -*-
#	Trimming a FASTQ file.
#	Author - Janu Verma
#	jv367@cornell.edu

import string

baseQdict = {x:ord(x) for x in string.printable}

class Trimming:
	"""
	Trimming a FASTQ sequence. 

	Parameters
	----------
	sequence : The sequence to be trimmed.
	qualities : Base qualities of the bp's in the sequence. 
	"""
	def __init__(self, sequence, qualities):
		self.seq = sequence
		self.qual = qualities


	def lowQualTrim(self, qualCutOff):
		"""
		Trims a sequence by removing low quality bp's. 

		Parameters
		----------
		qualCutOff : The threshold for the quality. The bases with quality below this threshold
					will be removed.

		Returns
		-------
		Trimmed sequence.			
		"""	
		trimmedSeq = ''
		l = len(self.seq)
		if (l != len(self.qual)):
			print "Number of base pairs is not same as the number of quality values."
			return None
		else:
			for i in range(l):
				qScore = baseQdict[self.qual[i]]
				qScore = qScore - 33
				if (qScore >= qualCutOff):
					trimmedSeq += self.seq[i]
			return trimmedSeq 	


	def mott(self, limitValue):
		"""
		Trims a sequence by using Mott's algorithm.

		Parameters
		----------
		limitValue : limiting value. 

		Returns
		------
		Trimmed sequence.
		"""	
		seq = self.seq
		qual = self.qual

		l = len(seq)
		totalLMP = 0.0
		oldLmp = 0.0
		i = 0
		trimmedSeq = ''
		while (i < l):
			qScore = baseQdict[qual[-i-1]]
			qScore -= 33
			expo = float(qScore)/10
			pError = 10**(-expo)
			lmpValue = limitValue - pError
			oldLmp = totalLMP
			totalLMP += lmpValue
			if (totalLMP <= 0.0):
				totalLMP = 0.0
				i +=1 
			else:
				trimmedSeq = seq[-i-1] + trimmedSeq
			if (oldLmp > totalLMP):
				break 	
			i += 1	
		return trimmedSeq


	def trimSequence(self, interval):
		"""
		Trims the sequence from both sides based on the interval.

		Parameters
		----------
		interval : The interval containing the number of bp's to be trimmed from left and right side respectively.

		Returns
		-------
		Trimmed sequence.			
		"""
		sequence =  self.seq
		start,end = interval
		n = len(sequence)
		return sequence[start: -end]	






				
						

 

			





























# -*- coding: utf-8 -*-
#	FASTQ to FASTA
#	Author - Janu Verma
#	jv367@cornell.edu

import sys


input_file = open(sys.argv[1])

	
class Fastq2Fasta:
	"""
	Converts a FASTQ file to a FASTA. 

	Paramaters
	----------
	input_file : A FASTQ file. 
	"""
	def __init__(self, input_file):
		self.fastq = input_file

	def conversion(self):
		"""
		Do the conversion.
		"""

		i = 1
		name, seq, baseQ = None, [], []
		for line in self.fastq:
			if (line.startswith("@")) and (i%4 != 0):
				if name: yield (name, ''.join(seq), ''.join(baseQ))
				name, seq, baseQ = line, [], []
			if (line[0] in ['A', 'G', 'T', 'C', 'N']):
				seq.append(line)
			if (i%4 == 0):
				baseQ.append(line)
			i += 1
		if name: yield (name, ''.join(seq), ''.join(baseQ))

	def fasta(self):
		"""
		Prints the output FASTA file.
		"""
		for name,seq,baseQ in self.conversion():
			print '>'+name[1:], seq.strip()


out = Fastq2Fasta(input_file)
out.fasta()
		

















